#!/bin/bash
#
# build_lapack.sh 
#

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e10|e14|e15> <prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && pwd -P`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

function setup_and_verify()
{
    # this should not complain
    echo "Finished building ${package} ${pkgver}"
    setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || ssi_die "setup_and_verify failed to set up the product."
    echo "${package} is installed at ${LAPACK_FQ_DIR}"
}

function make_osx_prodct_stub()
{
    mkdir -p ${pkgdir}/lib64 || ssi_die "Unable to make ${pkgdir}/lib64 for links."
    cd ${pkgdir}/lib64 || ssi_die "Unable to cd to ${pkgdir}/lib64 to make links."
#create softlinks here
    blas_path=/System/Library/Frameworks/Accelerate.framework/Versions/Current/Frameworks/vecLib.framework/Versions/A/libBLAS.dylib
    lapack_path=/System/Library/Frameworks/Accelerate.framework/Versions/Current/Frameworks/vecLib.framework/Versions/A/libLAPACK.dylib

    ln -sv ${blas_path} libblas.dylib || ssi_die "Unable to link BLAS library from ${blas_path}"
    ln -sv ${lapack_path} liblapack.dylib || ssi_die "Unable to link LAPACK library from ${lapack_path}"

    ${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || ssi_die "Unable to declare product."

    setup_and_verify

    # this must be last
    if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib64 ]
    then
       ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=lapack
origpkgver=v3_7_1
pkgver=${origpkgver}
ssibuildshims_version=v1_03_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' -e 's/^v//'`
pkgtar=lapack-${pkgdotver}.tgz
make_tarball_opts=("\${product_dir}" "\${package}" "\${pkgver}" "\${fullqual}")

get_this_dir
get_ssibuildshims
source define_basics NO_DEBUG_SOURCE

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib64 ]
then
   eval ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball "${make_tarball_opts[@]}"
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# make a product stub for OSX
if [ "${OS1}" = "Darwin" ]
then
    make_osx_prodct_stub
    exit 0
fi

# Nothing below is run on OSX...

rm -rf ${pkgdir}
mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
rm -rf ${blddir}
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
rm -rf ${sourcedir}
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# lapack's makefiles can not do an out-of-source build.
cd ${sourcedir} || ssi_die "Unable to cd to ${sourcedir}"
echo "In code directory: ${PWD}"
tar xf ${tardir}/${pkgtar} || ssi_die "Unable to unwind ${tardir}/${pkgtar} into ${PWD}"

# Apply the patch.
#osname=`uname -s`
#patch -p1 < ${patchdir}/${osname}.patch || ssi_die "Failed to apply patch"

# We only make an optimized build, because lapack does not support
# anything else (and nobody would really want it anyway).
cd ${sourcedir}/lapack-${pkgdotver} || ssi_die "Unable to cd to ${sourcedir}/lapack-${pkgdotver}"


# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake "${product_dir}" || ssi_ssi_die "failed to setup cmake"

set -x

cd ${blddir} || ssi_die "Unable to cd to ${blddir}"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

# We would wish to use -std=2008ts, but LAPACK source code doesn't
# support it. Fortunately it doesn't tickle anything that would make it
# binary incompatible with anything we would do downstream.
FC=gfortran \
CC=gcc \
CXX=FALSE \
    cmake \
    -DCMAKE_Fortran_FLAGS="-ffree-line-length-none" \
    -DCMAKE_INSTALL_PREFIX=${LAPACK_FQ_DIR} \
    -DCMAKE_BUILD_TYPE=RELEASE \
    -DBUILD_SHARED_LIBS=ON \
    -DCBLAS=ON \
    ${sourcedir}/lapack-${pkgdotver} || ssi_die "Failed to run CMake"
make -j $ncore || ssi_die "Failed in 'make'"
ctest -j $ncore || ssi_die "Failed in ctest"
make -j $ncore install || ssi_die "Failed to install"
# The Ubuntu build likes to use 'lib' as the subdirectory of
# $LAPACK_FQ_DIR, while SLF likes to use lib64. We'll use lib64.
if [ -f ${LAPACK_FQ_DIR}/lib/liblapack.so ]
then
  mkdir -p ${LAPACK_FQ_DIR}/lib64
  mv ${LAPACK_FQ_DIR}/lib/* ${LAPACK_FQ_DIR}/lib64/
  rm -r ${LAPACK_FQ_DIR}/lib
fi

set +x

[[ -f "${LAPACK_LIB}/liblapack.so.${pkgdotver}" ]] || ssi_die "failed to populate ${LAPACK_LIB} \n" \
  "       Something is very wrong"

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

setup_and_verify

# fix paths for pkgconfig
pkgcnf=`ls ${LAPACK_FQ_DIR}/lib64/pkgconfig/*.pc`
for file in ${pkgcnf}
do
  mv ${file} ${file}.bak
  cat ${file}.bak | sed -e "s%${LAPACK_FQ_DIR}%\${LAPACK_FQ_DIR}%g" > ${file}
done

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${LAPACK_LIB} ]
then
   eval ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball "${make_tarball_opts[@]}"
fi

exit 0
